/// <reference types="node" />
import type { StrictUnion } from 'simplytyped'
/**
 * If the variable `GITHUB_ACCESS_TOKEN` exists, use that according to:
 * https://developer.github.com/v3/#oauth2-token-sent-as-a-parameter
 */
interface GitHubToken {
	GITHUB_ACCESS_TOKEN: string
}
/**
 * If the variables `GITHUB_CLIENT_ID` and `GITHUB_CLIENT_SECRET` exist, use that according to:
 * https://developer.github.com/v3/#oauth2-keysecret
 */
interface GitHubClient {
	GITHUB_CLIENT_ID: string
	GITHUB_CLIENT_SECRET: string
}
/**
 * Provide `GITHUB_ACCESS_TOKEN` or a combination of `GITHUB_CLIENT_ID` and `GITHUB_CLIENT_SECRET`
 * https://developer.github.com/v3/#authentication
 */
export declare type GitHubCredentials = StrictUnion<GitHubToken | GitHubClient>
/** GitHub credentialed environment */
export declare type GitHubEnv = NodeJS.ProcessEnv & GitHubCredentials
/**
 * Check whether or not sufficient github credentials were supplied
 * @param credentials If no credentials were passed, then the environment variables are used if they exist.
 * @returns `true` if valid, otherwise throws if invalid
 */
export declare function validate(credentials?: GitHubCredentials): boolean
/**
 * Fetch the GitHub Auth Query String.
 * @param credentials If no credentials were passed, then the environment variables are used if they exist.
 * @returns The query string that you should append to your github API request url. Will be an empty string if no valid credentials were provided.
 */
export declare function getParams(credentials?: GitHubCredentials): string
export declare const githubQueryString: typeof getParams
export declare const fetch: typeof getParams
export default getParams
/**
 * Fetch the GitHub Authorization Header.
 * @param credentials If no credentials were passed, then the environment variables are used if they exist.
 * @returns The Authorization header to attach to the request to the GitHub request.
 */
export declare function getAuthHeader(credentials?: GitHubCredentials): string
export declare const githubAuthorizationHeader: typeof getAuthHeader
/**
 * Fetch the headers to attach to the request to the GitHub API
 * @param credentials If no credentials were passed, then the environment variables are used if they exist.
 * @returns The headers to attach to the request to the GitHub request.
 */
export declare function getHeaders(
	credentials?: GitHubCredentials
): {
	Accept: string
	Authorization: string
}
/**
 * Redact any github credentials from a string.
 * @param value The string to redact credentials from.
 * @returns The string with the credentials redacted
 */
export declare function redactParams(value: string): string
export declare const redact: typeof redactParams
//# sourceMappingURL=index.d.ts.map
