'use strict'
Object.defineProperty(exports, '__esModule', { value: true })
exports.redact = exports.redactParams = exports.getHeaders = exports.githubAuthorizationHeader = exports.getAuthHeader = exports.fetch = exports.githubQueryString = exports.getParams = exports.validate = void 0
/**
 * Check whether or not sufficient github credentials were supplied
 * @param credentials If no credentials were passed, then the environment variables are used if they exist.
 * @returns `true` if valid, otherwise throws if invalid
 */
function validate(credentials = process?.env) {
	if (
		credentials.GITHUB_ACCESS_TOKEN ||
		(credentials.GITHUB_CLIENT_ID && credentials.GITHUB_CLIENT_SECRET)
	) {
		return true
	} else {
		throw new Error(
			'missing github credentials; provide `GITHUB_ACCESS_TOKEN` or a combination of `GITHUB_CLIENT_ID` and `GITHUB_CLIENT_SECRET`'
		)
	}
}
exports.validate = validate
/**
 * Fetch the GitHub Auth Query String.
 * @param credentials If no credentials were passed, then the environment variables are used if they exist.
 * @returns The query string that you should append to your github API request url. Will be an empty string if no valid credentials were provided.
 */
function getParams(credentials = process?.env) {
	if (credentials.GITHUB_ACCESS_TOKEN) {
		return `access_token=${credentials.GITHUB_ACCESS_TOKEN}`
	} else if (credentials.GITHUB_CLIENT_ID && credentials.GITHUB_CLIENT_SECRET) {
		return `client_id=${credentials.GITHUB_CLIENT_ID}&client_secret=${credentials.GITHUB_CLIENT_SECRET}`
	} else {
		throw new Error('missing github credentials for query string')
	}
}
exports.getParams = getParams
// Aliases
exports.githubQueryString = getParams
exports.fetch = getParams
exports.default = getParams
/**
 * Fetch the GitHub Authorization Header.
 * @param credentials If no credentials were passed, then the environment variables are used if they exist.
 * @returns The Authorization header to attach to the request to the GitHub request.
 */
function getAuthHeader(credentials = process?.env) {
	if (credentials.GITHUB_ACCESS_TOKEN) {
		return `token ${credentials.GITHUB_ACCESS_TOKEN}`
	} else if (credentials.GITHUB_CLIENT_ID && credentials.GITHUB_CLIENT_SECRET) {
		return `Basic ${credentials.GITHUB_CLIENT_ID}:${credentials.GITHUB_CLIENT_SECRET}`
	} else {
		throw new Error('missing github credentials for authorization header')
	}
}
exports.getAuthHeader = getAuthHeader
// Aliases
exports.githubAuthorizationHeader = getAuthHeader
/**
 * Fetch the headers to attach to the request to the GitHub API
 * @param credentials If no credentials were passed, then the environment variables are used if they exist.
 * @returns The headers to attach to the request to the GitHub request.
 */
function getHeaders(credentials = process?.env) {
	return {
		Accept: 'application/vnd.github.v3+json',
		Authorization: getAuthHeader(credentials),
	}
}
exports.getHeaders = getHeaders
/**
 * Redact any github credentials from a string.
 * @param value The string to redact credentials from.
 * @returns The string with the credentials redacted
 */
function redactParams(value) {
	return value.replace(
		/(&?)(access_token|client_id|client_secret)=\w+/gi,
		'$1$2=REDACTED'
	)
}
exports.redactParams = redactParams
// Alias
exports.redact = redactParams
